package com.hywa.banktest.bankframework.schedule;

import java.io.File;
import java.util.*;

import com.alibaba.fastjson.JSONObject;
import com.hywa.banktest.bankframework.utils.DateUtils;
import com.hywa.banktest.bankframework.utils.string.StringUtil;
import com.hywa.banktest.entity.ClearingRecordDetail;
import com.hywa.banktest.entity.PaymentLog;
import com.hywa.banktest.entity.PaymentRecord;
import com.hywa.banktest.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import com.alibaba.fastjson.JSON;
import com.hywa.banktest.bankframework.pack.Dlmaps;
import com.hywa.banktest.bankframework.pack.TransVo;
import com.hywa.banktest.bankframework.service.BankServiceImpl;
import com.hywa.banktest.bankframework.utils.FtpUtil;
import com.hywa.banktest.bankframework.utils.TransUtils;
import com.hywa.banktest.model.OrderInfo;
import com.jfinal.kit.JsonKit;
import org.springframework.transaction.annotation.Transactional;

@Slf4j
@Component
public class TransSchedule {

	private static final String TRANS_SUCCESS_CODE="0000";

	@Autowired
	OrderInfoService orderInfoService;
	@Autowired
	private PaymentRecordService paymentRecordService;
	@Autowired
	BankServiceImpl bankService;
	@Autowired
	private PaymentLogService paymentLogService;
	@Autowired
	private ClearingRecordService clearingRecordService;

	@Autowired
	FtpUtil ftpUtil;
	@Value("${money.clear.ftp.retDir}")
	private String ftpDownloadDir;
	@Value("${money.clear.file.base.retdir}")
	private String baseDir;


	/**
	 * 资金清算定时任务
	 */
	//@Scheduled(cron = "0 */15 * * * ?")
	//@Scheduled(cron = "0 15 18 ? * *")
	@Transactional
	public void ClearingToBank(List<ClearingRecordDetail> list){
		log.info("开始更新订单状态为:转账中");
		if(null!=list && list.size()>0){
			for(ClearingRecordDetail bean:list){
				Map tmp = new HashMap();
				tmp.put("farmerId",bean.getFkFarmerId());
				tmp.put("orderStatus",4);
				tmp.put("transCode",null);
				tmp.put("transMsg",null);
				orderInfoService.updateOrderInfoByFarmer(tmp);
			}
			//TODO 转换为资金清算服务的列表信息
			log.info("开始转换为资金清算服务的列表信息");
			List<TransVo> transVos = JSON.parseArray(JsonKit.toJson(list), TransVo.class);
			log.info("transVos====================>{}",transVos);
			Dlmaps moneyClearResult = bankService.moneyClearing(transVos,null);
			log.info("moneyClearResult====================>{}",moneyClearResult);
			PaymentLog paymentLog = new PaymentLog();
			paymentLog.setResponesStatus(moneyClearResult.getOUT().getRET_CODE());
			paymentLog.setResponesMsg(moneyClearResult.getOUT().getMSG());
			paymentLog.setPaymentNum(Integer.valueOf(moneyClearResult.getIN().getTRADE_NUM()));
			paymentLog.setPaymentMoney(Float.valueOf(moneyClearResult.getIN().getTRADE_TOTAL_MONEY()));
			paymentLog.setCreateTime(new Date());
			// TODO 开始记录打款日志
			paymentLogService.insertSelectiveModel(paymentLog);
			if (!moneyClearResult.getOUT().getRET_CODE().equals("0000")) {
				throw new RuntimeException("资金清算请求失败");
			}
			//TODO 更新订单清算文件至订单
			for(ClearingRecordDetail bean : list){
				Map tmp = new HashMap();
				tmp.put("farmerId", bean.getFkFarmerId());
				tmp.put("transFile", moneyClearResult.getIN().getTRADE_FILE());
				orderInfoService.updateOrderInfoByFarmer(tmp);
			}
		}
	}

	/**
	 * 尽量在晚上22:00-01:00这段时间不要做，银行这时候在日终，交易时间跟自然时间不同。
	 * @return
	 */
	private static boolean isBankServiceTime() {
		Calendar cal = Calendar.getInstance();
		if (22<=cal.get(Calendar.HOUR_OF_DAY) && cal.get(Calendar.HOUR_OF_DAY)<=24) {
			return false;
		}
		else if (cal.get(Calendar.HOUR_OF_DAY) <= 1) {
			return false;
		}
		else {
			return true;
		}
	}
	

//	@Scheduled(cron = "0 0/5 * * * ?")
	//@Scheduled(cron = "0 0/10 0 * * ?")
	//@Scheduled(cron = "0 43 09 * * ?")
	//@Scheduled(cron = "0 0/3 * * * ?")
	public void checkClearOther(){
		log.info("--->开始调度文件了啊");
		String checkTime = DateUtils.formatDate(new Date(),DateUtils.DEFAULT_FORMAT);
		//得到转账中oderInfo的转账文件
		List<String> transFiles = clearingRecordService.getBankClearingFilesByTimed(checkTime);
		log.info("------>有没有文件"+ JSONObject.toJSONString(transFiles));
		if(null!=transFiles && transFiles.size()>0){
			for (String transFile : transFiles) {
				//回盘文件名称
				String transRetFile = transFile.replaceFirst("qut", "res");
				//如果邮储已经回盘则执行回盘业务处理
				if (ftpUtil.existFile(ftpDownloadDir, transRetFile)) {
					//回盘文件下载到本地
					ftpUtil.downloadFile(ftpDownloadDir, transRetFile, baseDir + File.separator + transRetFile);
					List<TransVo> trans = TransUtils.fileToTransVo(baseDir, transRetFile);
					bankService.updateOrderInfoPaymentRec(trans,transRetFile,transFile);
				}
			}
		}
	}

	@Scheduled(cron = "0 0/5 5-23 * * ?")
	public void checkClearFile(){
		log.info("--->开始调度文件了啊");
		String checkTime = DateUtils.formatDate(new Date(),DateUtils.DEFAULT_FORMAT);
		//得到转账中oderInfo的转账文件
		List<String> transFiles = clearingRecordService.getBankClearingFilesByTimed(checkTime);
		log.info("------>有没有文件"+ JSONObject.toJSONString(transFiles));

		List<String> resFiles = new ArrayList<>();
		if(null!=transFiles && transFiles.size()>0){
			for (String transFile : transFiles) {
				//回盘文件名称
				String transRetFile = transFile.replaceFirst("qut", "res");
				resFiles.add(transRetFile);
			}
		}
		//邮储已经回盘文件下载到本地
		List<String> localNewFiles = ftpUtil.localNewFiles(ftpDownloadDir, resFiles, baseDir + File.separator);
		log.info("------>本次新回盘文件"+ JSONObject.toJSONString(localNewFiles));

		//执行回盘业务处理
		if (localNewFiles != null && localNewFiles.size() > 0) {
			for (String newFile : localNewFiles){
				List<TransVo> trans = TransUtils.fileToTransVo(baseDir, newFile);
				bankService.updateOrderInfoPaymentRec(trans,newFile,newFile.replaceFirst("res", "qut"));
			}
		}
		log.info("结束调度回盘任务");
	}

	public String getClearText(String fileName){
		if (StringUtil.isEmpty(fileName)){
			return null;
		}
		//回盘文件名称
		String transRetFile = fileName.replaceFirst("qut", "res");
		//如果邮储已经回盘则执行回盘业务处理
		if (ftpUtil.existFile(ftpDownloadDir, transRetFile)) {
			//回盘文件下载到本地
			ftpUtil.downloadFile(ftpDownloadDir, transRetFile, baseDir + File.separator + transRetFile);
			return TransUtils.getClearText(baseDir, transRetFile);
		}
		return null;
	}

	//@Scheduled(cron = "0 0/2 * * * ?")
	public void checkClearOtherText(){
		log.info("--->手动解析回盘文件");
		//String checkTime = DateUtils.formatDate(new Date(),DateUtils.DEFAULT_FORMAT);
		//得到转账中oderInfo的转账文件
		//List<String> transFiles = clearingRecordService.getBankClearingFilesByTimed(checkTime);
		List<String> transFiles = new ArrayList<>();
		transFiles.add("20220616qut368.txt");
		/*transFiles.add("20180418qut006.txt");
		transFiles.add("20180418qut007.txt");
		transFiles.add("20220523qut203.txt");*/
		log.info("------>有没有文件"+ JSONObject.toJSONString(transFiles));
		List<String> resFiles = new ArrayList<>();
		if(null!=transFiles && transFiles.size()>0){
			for (String transFile : transFiles) {
				//回盘文件名称
				String transRetFile = transFile.replaceFirst("qut", "res");
				resFiles.add(transRetFile);
			}
		}
		//邮储已经回盘文件下载到本地
		List<String> localNewFiles = ftpUtil.localNewFiles(ftpDownloadDir, resFiles, baseDir + File.separator);

		//执行回盘业务处理
		if (localNewFiles != null && localNewFiles.size() > 0) {
			for (String newFile : localNewFiles){
				List<TransVo> trans = TransUtils.fileToTransVo(baseDir, newFile);
				bankService.updateOrderInfoPaymentRec(trans,newFile   ,newFile.replaceFirst("res", "qut"));
			}
		}
	}

	/**
	 * 手动解析回盘文件
	 */
	//@Scheduled(cron = "0 0/2 * * * ?")
	public void checkClearWriterText(){
		log.info("--->手动解析回盘文件");
		//邮储已经回盘文件下载到本地
		List<String> localNewFiles = new ArrayList<>();
		localNewFiles.add("20220616res368.txt");

		//执行回盘业务处理
		if (localNewFiles != null && localNewFiles.size() > 0) {
			for (String newFile : localNewFiles){
				List<TransVo> trans = TransUtils.fileToTransVo(baseDir, newFile);
				bankService.updateOrderInfoPaymentRec(trans,newFile   ,newFile.replaceFirst("res", "qut"));
			}
		}
	}
}
