package com.hywa.banktest.bankframework.utils.rsa;

import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.security.spec.AlgorithmParameterSpec;
import java.util.Arrays;

/**
 *
 * 版权所有：2016-贵州精英天成科技股份有限公司
 * 项目名称：plamsa-common
 *
 * 类描述：
 * 类名称：com.elite.common.utils.rsa.DES
 * 创建人： xiezt
 * 创建时间：2016年5月6日 下午5:10:30
 * 修改人：
 * 修改时间：2016年5月6日 下午5:10:30
 * 修改备注：
 * @version 1.0
 */
public class DES {

	// public static final String DES_ALGORITHM = "DES/CBC/PKCS5Padding";
	// 若采用NoPadding模式，data长度必须是8的倍数
	public static final String DES_ALGORITHM = "DES/CBC/NoPadding";

	private static Logger logger = LoggerFactory.getLogger(DES.class);

	/**
	 * DES-CBC模式加密
	 *
	 * @param plainData
	 *            明文
	 * @param keyData
	 *            密钥
	 * @param ivData
	 *            初始化向量
	 * @return 密文
	 */
	public static final byte[] encryptCBC(byte[] plainData, byte[] keyData, byte[] ivData) {
		byte[] encryptData = null;

		try {
			Cipher cipher = Cipher.getInstance(DES_ALGORITHM);
			AlgorithmParameterSpec paramSpec = new IvParameterSpec(ivData);
			SecretKey key = new SecretKeySpec(keyData, "DES");// key
			cipher.init(Cipher.ENCRYPT_MODE, key, paramSpec);

			int remainder = plainData.length%8;
			if (0!=remainder){
				int oldLen = plainData.length;
				plainData = Arrays.copyOf(plainData, oldLen + 8 - remainder);
				Arrays.fill(plainData,oldLen,plainData.length,(byte)0);
			}

			encryptData = cipher.doFinal(plainData);
		} catch (Exception e) {
			logger.error(JSONObject.toJSONString(e));
		}

		return encryptData;
	}

	/**
	 * DES-CBC模式解密
	 *
	 * @param ivData
	 *            密文
	 * @param keyData
	 *            密钥
	 * @param ivData
	 *            初始化向量
	 * @return 明文
	 */
	public static final byte[] decryptCBC(byte[] encryptData, byte[] keyData, byte[] ivData) {
		byte[] plainData = null;
		try {
			Cipher cipher = Cipher.getInstance(DES_ALGORITHM);
			AlgorithmParameterSpec paramSpec = new IvParameterSpec(ivData);
			SecretKey key = new SecretKeySpec(keyData, "DES");// key
			cipher.init(Cipher.DECRYPT_MODE, key, paramSpec);
			plainData = cipher.doFinal(encryptData);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error(JSONObject.toJSONString(e));
		}
		if (plainData != null){
			int len = plainData.length;
			for (int i = len;i>0;i--){
				if (plainData[i-1] == (byte)0){
					plainData = Arrays.copyOf(plainData, i-1);
				}else {
					break;
				}
			}

		}
		return plainData;
	}

}
